#include	"dv_ap.h"

extern	int node_response_state;
void	node_term (void);


void    sig_init(void)
{
        sigemptyset (&save_sigset);
        sigaddset (&save_sigset, SIGUSR1);
        sigaddset (&save_sigset, SIGUSR2);
        sigaddset (&save_sigset, SIGTERM);
        sigaddset (&save_sigset, SIGINT);
}

void	node_close (void)
{
        usb_reset (udev);
        usb_close (udev);
	node_term();
}

void	node_term (void)
{
	node_NoRespReply_sw = FALSE;
	node_response_state = 0;
	node_gw_resp_sw = FALSE;
	node_last_frame_sw = FALSE;
	dvap_voice_send_sw = FALSE;
	memset (&node_save_frame_id, 0x00, 2);
	if (node_sw)
	{
		time(&cur_time);
		fprintf (log_file, "%24.24s Node Adapter down.\n", ctime(&cur_time));
		fflush (log_file);
	}
	node_sw = FALSE;
}

void	dv_pkt_set(struct dv_packet *hdr)
{
	memcpy (hdr->id, "DSVT", 4);
	memset (hdr->filler, 0x00, 3);	
	hdr->dstar.b_bone.b_b.dest_repeater_id = 0x00;
	hdr->dstar.b_bone.b_b.send_repeater_id = 0x01;
	hdr->dstar.b_bone.b_b.send_terminal_id = 0x03;
}

int	cos_check(void)
{
	if (usb_control_msg(udev, 0xC0, GET_AD_STATUS, 0, 0, &status, 1, 100) < 0)
	{
		node_close();      /* node adapter down ? */
		return FALSE;
	}
	if (status & COS_OnOff) return TRUE;
	return FALSE;
}

int	htoi (const char *s)
{
	int n;

	if ( *s != '0' || !(*(s+1) != 'x' || *(s+1) != 'X') ) return 0;
    
	for (n=0,s+=2 ; *s ; s++)
	{
		if ( *s >= '0' && *s <= '9' ) 
		{
			n = 16 * n + (*s - '0');
		}
		else if ( *s >= 'a' && *s <= 'f' ) {
			n = 16 * n + ((*s - 'a') + 10);
		}
		else if ( *s >= 'A' && *s <= 'F' ) {
			n = 16 * n + ((*s - 'A') + 10);
		}
	}
    
	return n;
}

void	printOnOff (char sw)
{
	if (sw) fprintf (log_file, "ON\n");
	else	fprintf (log_file, "OFF\n");
}

void	putFifo (int len, struct dv_packet pkt)
{
	struct FifoPkt	*ret;

	ret = malloc (sizeof (struct FifoPkt));
	if (ret == NULL)
	{
		fprintf (log_file, "memory error\n");
		fflush (log_file);
		return;
	}
	ret->next = NULL;
	ret->length = len;
	memcpy (&ret->pkt, &pkt, len);
	Wp->next = ret;
	Wp = ret;
}

int	getFifo (int *len, struct dv_packet *pkt)
{
	struct	FifoPkt	*tmp;

	if (Rp->next == NULL) return FALSE;
	tmp = Rp;
	Rp = Rp->next;
	*len = Rp->length;
	memcpy (pkt, &Rp->pkt, Rp->length);
	free (tmp);
	return TRUE;
}

void	ReqPositionInfo (char call[])
{
	char ReqPosition[16];

	memset (ReqPosition, 0x00, 16);
	if (call[0] == '/')
	{
		memcpy (ReqPosition, &call[1], 6);
		ReqPosition[7] = call[7];
	}
	else
	{
		memcpy (&ReqPosition[8], call, 8);
	}

	ReqPosition[4] = 0x02;
	memcpy (&ReqPosition[8], call, 8);

	sendto(trust_sd, &ReqPosition, 16, 0,
		trust_sock->ai_addr, trust_sock->ai_addrlen);
}

void	read_trust(void)
{
	int	len;
	char	buff[40];
        in_addr_len = sizeof(struct sockaddr_storage);

        if((len = recvfrom(trust_sd, &buff, sizeof(buff), 0,
                (struct sockaddr *)&in_addr, &in_addr_len)) < 0)
        {
                time(&cur_time);
                fprintf (log_file, "%24.24s recvfrom error %s\n",
                        ctime(&cur_time), strerror(errno));
                fflush (log_file);
                return ;
        }
	if ((buff[4] == 0x02) && (buff[2] == 0x80) && (buff[3] == 0x00))
	{
		get_position_sw = TRUE;
		memcpy (gateway_position, buff, len);
	}
}

void	ReqPositionUpdate (struct dv_packet *pkt)
{
	char	temp[32];

	memset (temp, 0x00, 32);
	temp[4] = 0x01;
	memcpy (&temp[24], gateway_callsign, 8);

	//temp[15] = 0x20;
	memcpy (&temp[16], gateway_callsign, 8);
	temp[23] = 0x20;
	memcpy (&temp[8], pkt->dstar.b_bone.dstar_udp.rf_header.MyCall, 8);
	in_addr_len = sizeof(struct sockaddr_storage);
        sendto(trust_sd, &temp, 32, 0,
                trust_sock->ai_addr, trust_sock->ai_addrlen);
}

void	dv_log_send (struct dv_packet *pkt)
{
	char	log_temp[80];
	int	ret;

	struct	timeval	c_time;

	if (!memcmp (pkt->dstar.b_bone.dstar_udp.rf_header.RPT1Call,
		"DIRECT  ", 8) 
	|| !memcmp (pkt->dstar.b_bone.dstar_udp.rf_header.RPT2Call,
		"DIRECT  ", 8))
	return;

        memset (&hints, 0x00, sizeof(hints));
        hints.ai_socktype = SOCK_STREAM;
        hints.ai_family = PF_UNSPEC;
        sprintf (PORT, "%d", logd_port);
        if ((ret = getaddrinfo (trust_name, PORT, &hints, &logd_sock)) != 0)
        {
                time (&cur_time);
                fprintf (log_file, "%24.24s getaddrinfo error(logd sock) %s\n",
                        ctime(&cur_time), gai_strerror(ret));
                fprintf (log_file, "%24.24s trust : %s\n", ctime(&cur_time), trust_name);
                fflush (log_file);
        }
        if((logd_sd = socket(logd_sock->ai_family, logd_sock->ai_socktype, logd_sock->ai_protocol)) < 0)
        {
                fprintf (log_file, "%24.24s socket error in logd sock %s\n",
                        ctime(&cur_time), strerror(errno));
                fflush (log_file);
                return;
        }

	ret = connect (logd_sd, logd_sock->ai_addr, logd_sock->ai_addrlen);
        int nodelay_flag = 1;
        setsockopt(logd_sd, IPPROTO_TCP, TCP_NODELAY, (void*)
                                        &nodelay_flag, sizeof(int));
	memset (log_temp, 0x00, 80); 	
	memcpy (log_temp, "DSLG", 4);
	log_temp[7] = 0x01;
	memcpy (&log_temp[8], gateway_callsign, 8);
	gettimeofday (&c_time,  NULL);

	log_temp[16] = (c_time.tv_sec >> 24) & 0xff;
	log_temp[17] = (c_time.tv_sec >> 16) & 0xff;
	log_temp[18] = (c_time.tv_sec >> 8) & 0xff;
	log_temp[19] = c_time.tv_sec & 0xff;
	log_temp[20] = (c_time.tv_usec >> 24) & 0xff;
	log_temp[21] = (c_time.tv_usec >> 16) & 0xff;
	log_temp[22] = (c_time.tv_usec >> 8) & 0xff;
	log_temp[23] = c_time.tv_usec & 0xff;
	memcpy (&log_temp[24], 
		pkt->dstar.b_bone.dstar_udp.rf_header.MyCall, 8);
	memcpy (&log_temp[32], 
		pkt->dstar.b_bone.dstar_udp.rf_header.YourCall, 8);
	memcpy (&log_temp[48], 
		pkt->dstar.b_bone.dstar_udp.rf_header.RPT2Call, 8);
	memcpy (&log_temp[56], &gateway_position[16], 8);
	memcpy (&log_temp[64], 
		pkt->dstar.b_bone.dstar_udp.rf_header.RPT1Call, 8);
	memcpy (&log_temp[72], &gateway_position[24], 8);

	ret = send (logd_sd, log_temp, 80, 0);
	ret = close (logd_sd);
	freeaddrinfo (logd_sock);	

}

void    short_msg(struct aprs_msg *msg, char voice[])
{
	int	data_length;

        if ((voice[14] & 0x1f) % 2)
        {
                msg->tmp.mini_header = voice[24] ^ 0x70;
                msg->tmp.temp[0] = voice [25] ^ 0x4f;
                msg->tmp.temp[1] = voice [26] ^ 0x93;
        }
        else
        {
                msg->tmp.temp[2] = voice[24] ^ 0x70;
                msg->tmp.temp[3] = voice[25] ^ 0x4f;
                msg->tmp.temp[4] = voice[26] ^ 0x93;
                switch (msg->tmp.mini_header)
                {
                        case 0x40:      // short message
                                memcpy (&msg->short_msg[0], &msg->tmp.temp[0], 5);
                                break;
                        case 0x41:
                                memcpy (&msg->short_msg[5], &msg->tmp.temp[0], 5);
                                break;
                        case 0x42:
                                memcpy (&msg->short_msg[10], &msg->tmp.temp[0], 5);
                                break;
                        case 0x43:
                                memcpy (&msg->short_msg[15], &msg->tmp.temp[0], 5);
                                break;
			
			case 0x31:      // slow data
                        case 0x32:
                        case 0x33:
                        case 0x34:
                        case 0x35:
				data_length = msg->tmp.mini_header & 0x0f;
                                if (data_length > 5) data_length = 5;
                                memcpy (&msg->aprs_msg[msg->msg_pnt],
                                	&msg->tmp.temp[0], data_length);
                               	msg->msg_pnt += data_length;
                                if (msg->aprs_msg[msg->msg_pnt - 1] == 0x0d)
                                	msg->aprs_msg[msg->msg_pnt++] = 0x0a;
                                if (msg->aprs_msg[msg->msg_pnt - 1] == 0x0a)
                                {
                           		msg->aprs_msg[msg->msg_pnt] = 0x00;
                               		if (msg->msg_pnt > 6)
                                       	{
                                       		if (!strncmp(msg->aprs_msg, "$$CRC", 5))
                                                {
                                                	gps_a (msg->aprs_msg, msg);
                                                }
                                                else if (!strncmp(msg->aprs_msg, "$GP", 3))
                                                {
                                                	gps (msg->aprs_msg, msg);
                                                }
                                                else if (msg->aprs_msg[0] != 0x0a)
                                                {
                                                	gps (msg->aprs_msg, msg);
                                                }
                             		}
                                        msg->msg_pnt = 0;
                    		}
                                if (msg->msg_pnt > 240)
                                	msg->msg_pnt = 240;
                                break;
		}
	}
}

void    getOwnIp (void)
{
        struct  ifreq   ifr;
        int     fd;

        fd = socket (AF_INET, SOCK_DGRAM, 0);
        if (OwnDvApIP.s_addr == 0)
        {
                ifr.ifr_addr.sa_family = AF_INET;
                strncpy (ifr.ifr_name, NicDevice, IFNAMSIZ-1);
                ioctl (fd, SIOCGIFADDR, &ifr);
                close (fd);
                {
                        time(&cur_time);
                        fprintf(log_file, "%24.24s Own IP address %s\n", ctime(&cur_time),
                                inet_ntoa (((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr));
                        fflush (log_file);
                }
                OwnDvApIP.s_addr = ((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr;
        }
}

void	AddPortForward()
{
	char	cmd[128];

	sprintf (cmd, "upnpc -a %s %d %d udp", inet_ntoa(OwnDvApIP.s_addr),
			gateway_port, gateway_port);
	time(&cur_time);
	fprintf (log_file, "%24.24s %s\n", ctime(&cur_time), cmd);
	fflush (log_file);
	if (system (cmd) != 0)
	{
		fprintf (log_file, "%24.24s cmd error %s\n", ctime(&cur_time), cmd);
		fflush (log_file);
	}
}

void    DelPortForward()
{
        char    cmd[128];

        sprintf (cmd, "upnpc -d %d udp", gateway_port);
        time(&cur_time);
        fprintf (log_file, "%24.24s %s\n", ctime(&cur_time), cmd);
        fflush (log_file);
        if (system (cmd) != 0)
	{
		fprintf (log_file, "%24.24s cmd error %s\n", ctime(&cur_time), cmd);
		fflush (log_file);
	}
}

