#include	"dv_ap.h"

int	handler_init(void);
int     read_config(int argc, char **argv);
void	sig_init(void);

int	init(int argc, char **argv)
{
	int	err;

	sig_init();
	uname (&uname_buf);
	tzset();
	daemon (0, 0);

	sys_timeout.tv_sec = 60;
	sys_timeout.tv_usec = 0;
	node_NoRespReply_sw = FALSE;
	dvap_NoRespReply_sw = FALSE;
	HeaderLength = 0;
	VenderID = 0x4d8;
	ProductID = 0x300;

	get_position_sw = FALSE;
	
	node_sw = TRUE;
	dvap_sw = TRUE;
	dvap_freq = 0;
	dvap_tx_freq = 0;
	dvap_rx_freq = 0;
	dvap_squelch = 0;
	dvap_calibration = 0;
	dvap_auto_calibration = FALSE;
	dvap_auto_calibration_set = FALSE;
	dvap_squelch_status = 0x00;

	aprs_port = 0;
	aprs_send_interval = 0;
	send_check_pnt = NULL;
	time (&BeaconTime);

        tv.tv_sec = 0;
        tv.tv_nsec = 2000000;   /* 2 m sec. */
        FD_ZERO (&save_rfds);
	
	memset (node_area_rep_callsign, 0x20, 8);
	memset (dvap_area_rep_callsign, 0x20, 8);

/* LOG FILE open */
	log_file = fopen (LOG_FILE, "a");
	time (&cur_time);
	fprintf (log_file, "\n%24.24s dv_ap start V%s (Compiled %s %s)\n", ctime(&cur_time), PACKAGE_VERSION, __DATE__, __TIME__);
	fflush(log_file);

/* make PID file */
	pid_file = fopen (PID_FILE, "w");
	if (pid_file == NULL)
	{
		fprintf (log_file, "%24.24s Already running dv_ap.\n", 
			ctime(&cur_time));
		fflush (log_file);
	}
	fprintf (pid_file, "%d", getpid());
	fclose (pid_file);

	trust_port = TRUST_PORT;
	logd_port  = LOGD_PORT;
	gateway_port = GATEWAY_PORT;
	gateway_src_port = GATEWAY_SRC_PORT;
        memset (NicDevice, 0x00, IFNAMSIZ);
        strcpy (NicDevice, NicDeviceInitValue);
	upnp_sw = 1;

	if (!handler_init()) return FALSE;

	if (!read_config(argc, (char **)argv)) return FALSE;

	if (debug)
	{
		time (&cur_time);
		fprintf (log_file, "%24.24s node area repeater : %8.8s\n", 
			ctime(&cur_time), node_area_rep_callsign);
		fprintf (log_file, "%24.24s dvap area repeater : %8.8s\n",
			ctime(&cur_time), dvap_area_rep_callsign);
		fprintf (log_file, "%24.24s gateway call  : %8.8s\n", 
			ctime (&cur_time), gateway_callsign);
		fprintf (log_file, "%24.24s trust : %s\n",
			ctime (&cur_time), trust_name);
		fflush (log_file);
	}
	

	if (upnp_sw)
	{
		getOwnIp();
		AddPortForward();
	}


	voice_pnt = 0;
	seq = 0;
	node_last_frame_sw = FALSE;
	dvap_last_frame_sw = FALSE;
	rep_position_send_sw = FALSE;
	send_null_sw = FALSE;
	sig_term = FALSE;

	node_usb_init();
	dvap_open();

	memset (&hints, 0x00, sizeof(hints));
	hints.ai_socktype = SOCK_DGRAM;
	hints.ai_family = PF_UNSPEC;
	sprintf (PORT, "%d", trust_port);
	if ((err = getaddrinfo (trust_name, PORT, &hints, &trust_sock)) != 0)
	{
		time (&cur_time);
		fprintf (log_file, "%24.24s getaddrinfo error(trust sock) %s\n",
			ctime(&cur_time), gai_strerror(err));
		fprintf (log_file, "%24.24s trust : %s\n", ctime(&cur_time), trust_name);
		fflush (log_file);
	}
        if((trust_sd = socket(trust_sock->ai_family, trust_sock->ai_socktype, trust_sock->ai_protocol)) < 0)
        {
                fprintf (log_file, "%24.24s socket error in trust sock %s\n",
                        ctime(&cur_time), strerror(errno));
                fflush (log_file);
                return FALSE;
        }
	FD_SET (trust_sd, &save_rfds);
			
        memset (&hints, 0x00, sizeof(hints));
        hints.ai_socktype = SOCK_DGRAM;
        hints.ai_family = PF_UNSPEC;
        hints.ai_flags = AI_PASSIVE;
        sprintf (PORT, "%d", gateway_port);
        if ((err = getaddrinfo (NULL, PORT, &hints, &gateway_in)) != 0)
        {
                time (&cur_time);
                fprintf (log_file, "%24.24s getaddrinfo error(gateway in) %s\n",
                        ctime(&cur_time), gai_strerror(err));
                fprintf (log_file, "%24.24s trust : %s\n", ctime(&cur_time), trust_name);
                fflush (log_file);
        }
        if((in_sd = socket(gateway_in->ai_family, gateway_in->ai_socktype, gateway_in->ai_protocol)) < 0)
	{
		fprintf (log_file, "%24.24s socket error in gateway in %s\n",
			ctime(&cur_time), strerror(errno));
		fflush (log_file);
                return FALSE;
        }
	FD_SET (in_sd, &save_rfds);

	time(&cur_time); 
        int yes_flag = 1;
        setsockopt(in_sd, SOL_SOCKET, SO_REUSEADDR,
		(void *)&yes_flag, sizeof(int));
	if(bind(in_sd, gateway_in->ai_addr, gateway_in->ai_addrlen) < 0) 
	{
		fprintf (log_file, "%24.24s bind error in gateway port %s\n",
			ctime(&cur_time), strerror(errno));
		fflush (log_file);
		return FALSE;
	}

	memset(buf_pkt.buf, 0, sizeof(buf_pkt.buf));
 
	gettimeofday (&tm0, NULL);
	srand(tm0.tv_sec);
	tm0.tv_sec -= sys_timeout.tv_sec;
	tm0.tv_usec = sys_timeout.tv_usec;

	memset (node_save_frame_id, 0x00, 2);
	memset (dvap_save_frame_id, 0x00, 2);
	InetInTime.tv_sec = 0;
	InetInTime.tv_usec = 0;

	if (sys_timeout.tv_sec == 0) send_null_sw = TRUE;

	Rp = malloc (sizeof (struct FifoPkt));
	Wp = Rp;
	Rp->next = NULL;

	init_sw = FALSE;
	node_voice_send_sw = FALSE;
	dvap_voice_send_sw = FALSE;

	time (&re_check);

	return TRUE;
}

